@@ -53,6 +53,7 @@
 
 typedef struct {
   WebKitWebContext *web_context;
+  WebKitNetworkSession *network_session;
   EphyHistoryService *global_history_service;
   EphyGSBService *global_gsb_service;
   EphyEncodings *encodings;
@@ -78,6 +79,7 @@ enum {
   ALLOW_UNSAFE_BROWSING,
   RELOAD_PAGE,
   PASSWORD_FORM_FOCUSED,
+  PASSWORD_FORM_SUBMITTED,
 
   LAST_SIGNAL
 };
@@ -96,7 +98,7 @@ static EphyEmbedShell *embed_shell = NULL;
 
 static void ephy_embed_shell_tabs_catalog_iface_init (EphyTabsCatalogInterface *iface);
 
-G_DEFINE_TYPE_WITH_CODE (EphyEmbedShell, ephy_embed_shell, DZL_TYPE_APPLICATION,
+G_DEFINE_TYPE_WITH_CODE (EphyEmbedShell, ephy_embed_shell, ADW_TYPE_APPLICATION,
                          G_ADD_PRIVATE (EphyEmbedShell)
                          G_IMPLEMENT_INTERFACE (EPHY_TYPE_TABS_CATALOG,
                                                 ephy_embed_shell_tabs_catalog_iface_init))
@@ -147,7 +149,7 @@ tabs_catalog_get_tabs_info (EphyTabsCatalog *catalog)
   g_assert ((gpointer)catalog == (gpointer)embed_shell);
 
   windows = gtk_application_get_windows (GTK_APPLICATION (embed_shell));
-  database = webkit_web_context_get_favicon_database (ephy_embed_shell_get_web_context (embed_shell));
+  database = ephy_embed_shell_get_favicon_database (embed_shell);
 
   for (GList *l = windows; l && l->data; l = l->next) {
     tabs = ephy_embed_container_get_children (l->data);
@@ -198,6 +200,7 @@ ephy_embed_shell_dispose (GObject *object)
   g_clear_object (&priv->password_manager);
   g_clear_object (&priv->permissions_manager);
   g_clear_object (&priv->web_context);
+  g_clear_object (&priv->network_session);
   g_clear_pointer (&priv->guid, g_free);
   g_clear_object (&priv->filters_manager);
   g_clear_object (&priv->search_engine_manager);
@@ -207,7 +210,7 @@ ephy_embed_shell_dispose (GObject *object)
 
 static void
 web_process_extension_password_form_focused_message_received_cb (WebKitUserContentManager *manager,
-                                                                 WebKitJavascriptResult   *message,
+                                                                 JSCValue                 *message,
                                                                  EphyEmbedShell           *shell)
 {
   guint64 page_id;
@@ -215,7 +218,7 @@ web_process_extension_password_form_focused_message_received_cb (WebKitUserConte
   g_autoptr (GVariant) variant = NULL;
   g_autofree char *message_str = NULL;
 
-  message_str = jsc_value_to_string (webkit_javascript_result_get_js_value (message));
+  message_str = jsc_value_to_string (message);
   variant = g_variant_parse (G_VARIANT_TYPE ("(tb)"), message_str, NULL, NULL, NULL);
 
   g_variant_get (variant, "(tb)", &page_id, &insecure_form_action);
@@ -282,13 +285,13 @@ history_set_url_hidden_cb (EphyHistoryService *service,
 
 static void
 web_process_extension_overview_message_received_cb (WebKitUserContentManager *manager,
-                                                    WebKitJavascriptResult   *message,
+                                                    JSCValue                 *message,
                                                     EphyEmbedShell           *shell)
 {
   EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
   g_autofree char *url_to_remove = NULL;
 
-  url_to_remove = jsc_value_to_string (webkit_javascript_result_get_js_value (message));
+  url_to_remove = jsc_value_to_string (message);
 
   ephy_history_service_set_url_hidden (priv->global_history_service,
                                        url_to_remove, TRUE, NULL,
@@ -298,45 +301,45 @@ web_process_extension_overview_message_received_cb (WebKitUserContentManager *ma
 
 static void
 web_process_extension_tls_error_page_message_received_cb (WebKitUserContentManager *manager,
-                                                          WebKitJavascriptResult   *message,
+                                                          JSCValue                 *message,
                                                           EphyEmbedShell           *shell)
 {
   guint64 page_id;
 
-  page_id = jsc_value_to_double (webkit_javascript_result_get_js_value (message));
+  page_id = jsc_value_to_double (message);
   g_signal_emit (shell, signals[ALLOW_TLS_CERTIFICATE], 0, page_id);
 }
 
 static void
 web_process_extension_reload_page_message_received_cb (WebKitUserContentManager *manager,
-                                                       WebKitJavascriptResult   *message,
+                                                       JSCValue                 *message,
                                                        EphyEmbedShell           *shell)
 {
   guint64 page_id;
 
-  page_id = jsc_value_to_double (webkit_javascript_result_get_js_value (message));
+  page_id = jsc_value_to_double (message);
   g_signal_emit (shell, signals[RELOAD_PAGE], 0, page_id);
 }
 
 static void
 web_process_extension_unsafe_browsing_error_page_message_received_cb (WebKitUserContentManager *manager,
-                                                                      WebKitJavascriptResult   *message,
+                                                                      JSCValue                 *message,
                                                                       EphyEmbedShell           *shell)
 {
   guint64 page_id;
 
-  page_id = jsc_value_to_double (webkit_javascript_result_get_js_value (message));
+  page_id = jsc_value_to_double (message);
   g_signal_emit (shell, signals[ALLOW_UNSAFE_BROWSING], 0, page_id);
 }
 
 static void
 web_process_extension_about_apps_message_received_cb (WebKitUserContentManager *manager,
-                                                      WebKitJavascriptResult   *message,
+                                                      JSCValue                 *message,
                                                       EphyEmbedShell           *shell)
 {
   g_autofree char *app_id = NULL;
 
-  app_id = jsc_value_to_string (webkit_javascript_result_get_js_value (message));
+  app_id = jsc_value_to_string (message);
   ephy_web_application_delete (app_id, NULL);
 }
 
@@ -358,48 +361,6 @@ property_to_uint64 (JSCValue   *value,
   return (guint64)jsc_value_to_double (prop);
 }
 
-typedef struct {
-  EphyPasswordManager *password_manager;
-  EphyPermissionsManager *permissions_manager;
-  char *origin;
-  char *target_origin;
-  char *username;
-  char *password;
-  char *username_field;
-  char *password_field;
-  gboolean is_new;
-} SaveAuthRequest;
-
-static void
-save_auth_request_free (SaveAuthRequest *request)
-{
-  g_object_unref (request->password_manager);
-  g_object_unref (request->permissions_manager);
-  g_free (request->origin);
-  g_free (request->target_origin);
-  g_free (request->username);
-  g_free (request->password);
-  g_free (request->username_field);
-  g_free (request->password_field);
-  g_free (request);
-}
-
-static void
-save_auth_request_response_cb (gint             response_id,
-                               SaveAuthRequest *data)
-{
-  if (response_id == GTK_RESPONSE_REJECT) {
-    ephy_permissions_manager_set_permission (data->permissions_manager,
-                                             EPHY_PERMISSION_TYPE_SAVE_PASSWORD,
-                                             data->origin,
-                                             EPHY_PERMISSION_DENY);
-  } else if (response_id == GTK_RESPONSE_YES) {
-    ephy_password_manager_save (data->password_manager, data->origin, data->target_origin,
-                                data->username, data->password, data->username_field,
-                                data->password_field, data->is_new);
-  }
-}
-
 static void
 web_process_extension_password_manager_save_real (EphyEmbedShell *shell,
                                                   JSCValue       *value,
@@ -416,7 +377,7 @@ web_process_extension_password_manager_save_real (EphyEmbedShell *shell,
   gboolean is_new = jsc_value_to_boolean (is_new_prop);
   guint64 page_id = property_to_uint64 (value, "pageID");
   EphyWebView *view;
-  SaveAuthRequest *request;
+  EphyPasswordRequestData *request_data;
 
   /* Both origin and target origin are required. */
   if (!origin || !target_origin)
@@ -445,43 +406,42 @@ web_process_extension_password_manager_save_real (EphyEmbedShell *shell,
   if (!view)
     return;
 
+  /* Since we're automatically storing the password, no user interaction is required,
+  so we don't need to send the data anywhere */
   if (!is_request) {
     ephy_password_manager_save (priv->password_manager, origin, target_origin, username,
                                 password, username_field, password_field, is_new);
     return;
-  }
+  } else {
+    /* User interaction is required, so whatever is handling the EmbedShell should
+    handle the request + saving the password */
 
-  request = g_new (SaveAuthRequest, 1);
-  request->password_manager = g_object_ref (priv->password_manager);
-  request->permissions_manager = g_object_ref (priv->permissions_manager);
-  request->origin = g_steal_pointer (&origin);
-  request->target_origin = g_steal_pointer (&target_origin);
-  request->username = g_steal_pointer (&username);
-  request->password = g_steal_pointer (&password);
-  request->username_field = g_steal_pointer (&username_field);
-  request->password_field = g_steal_pointer (&password_field);
-  request->is_new = is_new;
-  ephy_web_view_show_auth_form_save_request (view, request->origin, request->username,
-                                             (EphyPasswordSaveRequestCallback)save_auth_request_response_cb,
-                                             request, (GDestroyNotify)save_auth_request_free);
+    request_data = g_new (EphyPasswordRequestData, 1);
+    request_data->origin = g_steal_pointer (&origin);
+    request_data->target_origin = g_steal_pointer (&target_origin);
+    request_data->username = g_steal_pointer (&username);
+    request_data->password = g_steal_pointer (&password);
+    request_data->usernameField = g_steal_pointer (&username_field);
+    request_data->passwordField = g_steal_pointer (&password_field);
+    request_data->isNew = is_new;
+    g_signal_emit (shell, signals[PASSWORD_FORM_SUBMITTED], 0, request_data);
+  }
 }
 
 static void
 web_process_extension_password_manager_save_received_cb (WebKitUserContentManager *manager,
-                                                         WebKitJavascriptResult   *message,
+                                                         JSCValue                 *message,
                                                          EphyEmbedShell           *shell)
 {
-  JSCValue *value = webkit_javascript_result_get_js_value (message);
-  web_process_extension_password_manager_save_real (shell, value, FALSE);
+  web_process_extension_password_manager_save_real (shell, message, FALSE);
 }
 
 static void
 web_process_extension_password_manager_request_save_received_cb (WebKitUserContentManager *manager,
-                                                                 WebKitJavascriptResult   *message,
+                                                                 JSCValue                 *message,
                                                                  EphyEmbedShell           *shell)
 {
-  JSCValue *value = webkit_javascript_result_get_js_value (message);
-  web_process_extension_password_manager_save_real (shell, value, TRUE);
+  web_process_extension_password_manager_save_real (shell, message, TRUE);
 }
 
 static void
@@ -764,9 +724,9 @@ initialize_web_process_extensions (WebKitWebContext *web_context,
   gboolean private_profile;
 
 #if DEVELOPER_MODE
-  webkit_web_context_set_web_extensions_directory (web_context, BUILD_ROOT "/embed/web-process-extension");
+  webkit_web_context_set_web_process_extensions_directory (web_context, BUILD_ROOT "/embed/web-process-extension");
 #else
-  webkit_web_context_set_web_extensions_directory (web_context, EPHY_WEB_PROCESS_EXTENSIONS_DIR);
+  webkit_web_context_set_web_process_extensions_directory (web_context, EPHY_WEB_PROCESS_EXTENSIONS_DIR);
 #endif
 
   private_profile = priv->mode == EPHY_EMBED_SHELL_MODE_PRIVATE || priv->mode == EPHY_EMBED_SHELL_MODE_INCOGNITO || priv->mode == EPHY_EMBED_SHELL_MODE_AUTOMATION;
@@ -777,7 +737,7 @@ initialize_web_process_extensions (WebKitWebContext *web_context,
                              private_profile,
                              FALSE /* is_webextension */,
                              "" /* webextension_translations */);
-  webkit_web_context_set_web_extensions_initialization_user_data (web_context, g_steal_pointer (&user_data));
+  webkit_web_context_set_web_process_extensions_initialization_user_data (web_context, g_steal_pointer (&user_data));
 }
 
 static void
@@ -795,38 +755,39 @@ initialize_notification_permissions (WebKitWebContext *web_context,
   webkit_web_context_initialize_notification_permissions (web_context, permitted_origins, denied_origins);
 }
 
+static void
+ephy_embed_shell_create_network_session_if_needed (EphyEmbedShell *shell)
+{
+  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
+
+  if (priv->mode == EPHY_EMBED_SHELL_MODE_AUTOMATION)
+    priv->network_session = g_object_ref (webkit_web_context_get_network_session_for_automation (priv->web_context));
+  else if (priv->mode == EPHY_EMBED_SHELL_MODE_INCOGNITO)
+    priv->network_session = webkit_network_session_new_ephemeral ();
+  else {
+    priv->network_session = webkit_network_session_new (ephy_profile_dir (), ephy_cache_dir ());
+    webkit_network_session_set_persistent_credential_storage_enabled (priv->network_session, FALSE);
+  }
+
+  webkit_network_session_set_itp_enabled (priv->network_session,
+                                          g_settings_get_boolean (EPHY_SETTINGS_WEB,
+                                                                  EPHY_PREFS_WEB_ENABLE_ITP));
+}
+
 static void
 ephy_embed_shell_create_web_context (EphyEmbedShell *shell)
 {
   EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
-  g_autoptr (WebKitWebsiteDataManager) manager = NULL;
 
-  if (priv->mode == EPHY_EMBED_SHELL_MODE_INCOGNITO || priv->mode == EPHY_EMBED_SHELL_MODE_AUTOMATION) {
-    manager = webkit_website_data_manager_new_ephemeral ();
-  } else {
-    manager = webkit_website_data_manager_new ("base-data-directory", ephy_profile_dir (),
-                                               "base-cache-directory", ephy_cache_dir (),
-                                               NULL);
-    webkit_website_data_manager_set_persistent_credential_storage_enabled (manager, FALSE);
-  }
-
-  webkit_website_data_manager_set_itp_enabled (manager,
-                                               g_settings_get_boolean (EPHY_SETTINGS_WEB,
-                                                                       EPHY_PREFS_WEB_ENABLE_ITP));
-
-  priv->web_context = g_object_new (WEBKIT_TYPE_WEB_CONTEXT,
-                                    "website-data-manager", manager,
-                                    "process-swap-on-cross-site-navigation-enabled", TRUE,
-                                    NULL);
+  priv->web_context = webkit_web_context_new ();
 
   if (priv->mode == EPHY_EMBED_SHELL_MODE_AUTOMATION)
     webkit_web_context_set_automation_allowed (priv->web_context, TRUE);
 }
 
 static void
-download_started_cb (WebKitWebContext *web_context,
-                     WebKitDownload   *download,
-                     EphyEmbedShell   *shell)
+download_started_cb (EphyEmbedShell *shell,
+                     WebKitDownload *download)
 {
   EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
   g_autoptr (EphyDownload) ephy_download = NULL;
@@ -873,33 +834,10 @@ enable_itp_setting_changed_cb (GSettings      *settings,
                                EphyEmbedShell *shell)
 {
   EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
-  WebKitWebsiteDataManager *manager;
 
-  manager = webkit_web_context_get_website_data_manager (priv->web_context);
-  webkit_website_data_manager_set_itp_enabled (manager,
-                                               g_settings_get_boolean (EPHY_SETTINGS_WEB,
-                                                                       EPHY_PREFS_WEB_ENABLE_ITP));
-}
-
-static void
-update_system_scrollbars (EphyEmbedShell *shell)
-{
-  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
-  const char *theme_name;
-  gboolean enabled;
-
-  g_object_get (gtk_settings_get_default (),
-                "gtk-theme_name", &theme_name,
-                NULL);
-
-  /* Don't enable system scrollbars for Adwaita */
-  enabled = g_strcmp0 (theme_name, "Adwaita") &&
-            g_strcmp0 (theme_name, "Adwaita-dark") &&
-            g_strcmp0 (theme_name, "HighContrast") &&
-            g_strcmp0 (theme_name, "HighContrastInverse");
-
-  webkit_web_context_set_use_system_appearance_for_scrollbars (priv->web_context,
-                                                               enabled);
+  webkit_network_session_set_itp_enabled (priv->network_session,
+                                          g_settings_get_boolean (EPHY_SETTINGS_WEB,
+                                                                  EPHY_PREFS_WEB_ENABLE_ITP));
 }
 
 static void
@@ -907,15 +845,12 @@ ephy_embed_shell_startup (GApplication *application)
 {
   EphyEmbedShell *shell = EPHY_EMBED_SHELL (application);
   EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
-  g_autofree char *favicon_db_path = NULL;
+  WebKitWebsiteDataManager *data_manager;
   WebKitCookieManager *cookie_manager;
   g_autofree char *filename = NULL;
 
   G_APPLICATION_CLASS (ephy_embed_shell_parent_class)->startup (application);
 
-  webkit_web_context_set_process_model (priv->web_context, WEBKIT_PROCESS_MODEL_MULTIPLE_SECONDARY_PROCESSES);
-
-  webkit_web_context_set_sandbox_enabled (priv->web_context, TRUE);
   webkit_web_context_add_path_to_sandbox (priv->web_context, ephy_profile_dir (), TRUE);
   webkit_web_context_add_path_to_sandbox (priv->web_context, ephy_cache_dir (), TRUE);
   webkit_web_context_add_path_to_sandbox (priv->web_context, ephy_config_dir (), TRUE);
@@ -924,7 +859,7 @@ ephy_embed_shell_startup (GApplication *application)
   webkit_web_context_add_path_to_sandbox (priv->web_context, BUILD_ROOT, TRUE);
 #endif
 
-  g_signal_connect_object (priv->web_context, "initialize-web-extensions",
+  g_signal_connect_object (priv->web_context, "initialize-web-process-extensions",
                            G_CALLBACK (initialize_web_process_extensions),
                            shell, 0);
 
@@ -934,14 +869,8 @@ ephy_embed_shell_startup (GApplication *application)
 
   priv->password_manager = ephy_password_manager_new ();
 
-  /* Do not cache favicons in automation mode. Don't change the TLS policy either, since that's
-   * handled by session capabilities in automation mode.
-   */
-  if (priv->mode != EPHY_EMBED_SHELL_MODE_AUTOMATION) {
-    /* Favicon Database */
-    favicon_db_path = g_build_filename (ephy_cache_dir (), "icondatabase", NULL);
-    webkit_web_context_set_favicon_database_directory (priv->web_context, favicon_db_path);
-  }
+  data_manager = webkit_network_session_get_website_data_manager (priv->network_session);
+  webkit_website_data_manager_set_favicons_enabled (data_manager, TRUE);
 
   /* about: URIs handler */
   priv->about_handler = ephy_about_handler_new ();
@@ -974,28 +903,21 @@ ephy_embed_shell_startup (GApplication *application)
                                                          "ephy-resource");
 
   /* Store cookies in moz-compatible SQLite format */
-  cookie_manager = webkit_web_context_get_cookie_manager (priv->web_context);
-  if (!webkit_web_context_is_ephemeral (priv->web_context)) {
+  if (!webkit_network_session_is_ephemeral (priv->network_session)) {
+    cookie_manager = webkit_network_session_get_cookie_manager (priv->network_session);
     filename = g_build_filename (ephy_profile_dir (), "cookies.sqlite", NULL);
     webkit_cookie_manager_set_persistent_storage (cookie_manager, filename,
                                                   WEBKIT_COOKIE_PERSISTENT_STORAGE_SQLITE);
   }
 
-  g_signal_connect_object (priv->web_context, "download-started",
-                           G_CALLBACK (download_started_cb), shell, 0);
+  g_signal_connect_object (priv->network_session, "download-started",
+                           G_CALLBACK (download_started_cb), shell, G_CONNECT_SWAPPED);
 
   g_signal_connect_object (EPHY_SETTINGS_WEB, "changed::remember-passwords",
                            G_CALLBACK (remember_passwords_setting_changed_cb), shell, 0);
 
   g_signal_connect_object (EPHY_SETTINGS_WEB, "changed::enable-itp",
                            G_CALLBACK (enable_itp_setting_changed_cb), shell, 0);
-
-  update_system_scrollbars (shell);
-
-  g_signal_connect_swapped (gtk_settings_get_default (),
-                            "notify::gtk-theme-name",
-                            G_CALLBACK (update_system_scrollbars),
-                            shell);
 }
 
 static void
@@ -1054,6 +976,7 @@ ephy_embed_shell_constructed (GObject *object)
   priv->guid = g_dbus_generate_guid ();
 
   ephy_embed_shell_create_web_context (shell);
+  ephy_embed_shell_create_network_session_if_needed (shell);
 
   priv->permissions_manager = ephy_permissions_manager_new ();
   priv->filters_manager = ephy_filters_manager_new (NULL);
@@ -1083,8 +1006,7 @@ ephy_embed_shell_class_init (EphyEmbedShellClass *klass)
 
   object_properties[PROP_MODE] =
     g_param_spec_enum ("mode",
-                       "Mode",
-                       "The  global mode for this instance.",
+                       NULL, NULL,
                        EPHY_TYPE_EMBED_SHELL_MODE,
                        EPHY_EMBED_SHELL_MODE_BROWSER,
                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS);
@@ -1191,6 +1113,22 @@ ephy_embed_shell_class_init (EphyEmbedShellClass *klass)
                   G_TYPE_NONE, 2,
                   G_TYPE_UINT64,
                   G_TYPE_BOOLEAN);
+
+  /**
+   * EphyEmbedShell::password-form-submitted:
+   * @shell: the #EphyEmbedShell
+   * @data: An #EphyPasswordRequestData containing information regarding the password
+   *
+   * Emitted when a Password Save request has been received, and the UI should
+   * provide the user with a prompt of some form
+   */
+  signals[PASSWORD_FORM_SUBMITTED] =
+    g_signal_new ("password-form-submitted",
+                  EPHY_TYPE_EMBED_SHELL,
+                  G_SIGNAL_RUN_FIRST,
+                  0, NULL, NULL, NULL,
+                  G_TYPE_NONE, 1,
+                  G_TYPE_POINTER);
 }
 
 /**
@@ -1332,20 +1270,6 @@ ephy_embed_shell_get_mode (EphyEmbedShell *shell)
   return priv->mode;
 }
 
-/**
- * ephy_embed_shell_clear_cache:
- * @shell: an #EphyEmbedShell
- *
- * Clears the HTTP cache (temporarily saved web pages).
- **/
-void
-ephy_embed_shell_clear_cache (EphyEmbedShell *shell)
-{
-  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
-
-  webkit_web_context_clear_cache (priv->web_context);
-}
-
 EphyFiltersManager *
 ephy_embed_shell_get_filters_manager (EphyEmbedShell *shell)
 {
@@ -1370,6 +1294,14 @@ ephy_embed_shell_get_web_context (EphyEmbedShell *shell)
   return priv->web_context;
 }
 
+WebKitNetworkSession *
+ephy_embed_shell_get_network_session (EphyEmbedShell *shell)
+{
+  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
+
+  return priv->network_session;
+}
+
 EphyDownloadsManager *
 ephy_embed_shell_get_downloads_manager (EphyEmbedShell *shell)
 {
@@ -1406,6 +1338,16 @@ ephy_embed_shell_get_password_manager (EphyEmbedShell *shell)
   return priv->password_manager;
 }
 
+WebKitFaviconDatabase *
+ephy_embed_shell_get_favicon_database (EphyEmbedShell *shell)
+{
+  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
+  WebKitWebsiteDataManager *manager;
+
+  manager = webkit_network_session_get_website_data_manager (priv->network_session);
+  return webkit_website_data_manager_get_favicon_database (manager);
+}
+
 void
 ephy_embed_shell_register_ucm_handler (EphyEmbedShell           *shell,
                                        WebKitUserContentManager *ucm)
@@ -1414,54 +1356,58 @@ ephy_embed_shell_register_ucm_handler (EphyEmbedShell           *shell,
 
   /* User content manager */
   /* FIXME: See https://gitlab.gnome.org/GNOME/epiphany/-/issues/1664 */
-  webkit_user_content_manager_register_script_message_handler_in_world (ucm,
-                                                                        "overview",
-                                                                        priv->guid);
+  webkit_user_content_manager_register_script_message_handler (ucm,
+                                                               "overview",
+                                                               priv->guid);
   g_signal_connect_object (ucm, "script-message-received::overview",
                            G_CALLBACK (web_process_extension_overview_message_received_cb),
                            shell, 0);
 
   webkit_user_content_manager_register_script_message_handler (ucm,
-                                                               "tlsErrorPage");
+                                                               "tlsErrorPage",
+                                                               NULL);
   g_signal_connect_object (ucm, "script-message-received::tlsErrorPage",
                            G_CALLBACK (web_process_extension_tls_error_page_message_received_cb),
                            shell, 0);
 
   webkit_user_content_manager_register_script_message_handler (ucm,
-                                                               "reloadPage");
+                                                               "reloadPage",
+                                                               NULL);
   g_signal_connect_object (ucm, "script-message-received::reloadPage",
                            G_CALLBACK (web_process_extension_reload_page_message_received_cb),
                            shell, 0);
 
   webkit_user_content_manager_register_script_message_handler (ucm,
-                                                               "unsafeBrowsingErrorPage");
+                                                               "unsafeBrowsingErrorPage",
+                                                               NULL);
   g_signal_connect_object (ucm, "script-message-received::unsafeBrowsingErrorPage",
                            G_CALLBACK (web_process_extension_unsafe_browsing_error_page_message_received_cb),
                            shell, 0);
 
-  webkit_user_content_manager_register_script_message_handler_in_world (ucm,
-                                                                        "passwordFormFocused",
-                                                                        priv->guid);
+  webkit_user_content_manager_register_script_message_handler (ucm,
+                                                               "passwordFormFocused",
+                                                               priv->guid);
   g_signal_connect_object (ucm, "script-message-received::passwordFormFocused",
                            G_CALLBACK (web_process_extension_password_form_focused_message_received_cb),
                            shell, 0);
 
   webkit_user_content_manager_register_script_message_handler (ucm,
-                                                               "aboutApps");
+                                                               "aboutApps",
+                                                               NULL);
   g_signal_connect_object (ucm, "script-message-received::aboutApps",
                            G_CALLBACK (web_process_extension_about_apps_message_received_cb),
                            shell, 0);
 
-  webkit_user_content_manager_register_script_message_handler_in_world (ucm,
-                                                                        "passwordManagerSave",
-                                                                        priv->guid);
+  webkit_user_content_manager_register_script_message_handler (ucm,
+                                                               "passwordManagerSave",
+                                                               priv->guid);
   g_signal_connect_object (ucm, "script-message-received::passwordManagerSave",
                            G_CALLBACK (web_process_extension_password_manager_save_received_cb),
                            shell, 0);
 
-  webkit_user_content_manager_register_script_message_handler_in_world (ucm,
-                                                                        "passwordManagerRequestSave",
-                                                                        priv->guid);
+  webkit_user_content_manager_register_script_message_handler (ucm,
+                                                               "passwordManagerRequestSave",
+                                                               priv->guid);
   g_signal_connect_object (ucm, "script-message-received::passwordManagerRequestSave",
                            G_CALLBACK (web_process_extension_password_manager_request_save_received_cb),
                            shell, 0);
@@ -1491,21 +1437,25 @@ ephy_embed_shell_unregister_ucm_handler (EphyEmbedShell           *shell,
 {
   EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
 
-  webkit_user_content_manager_unregister_script_message_handler_in_world (ucm,
-                                                                          "overview",
-                                                                          priv->guid);
   webkit_user_content_manager_unregister_script_message_handler (ucm,
-                                                                 "tlsErrorPage");
+                                                                 "overview",
+                                                                 priv->guid);
   webkit_user_content_manager_unregister_script_message_handler (ucm,
-                                                                 "unsafeBrowsingErrorPage");
-  webkit_user_content_manager_unregister_script_message_handler_in_world (ucm,
-                                                                          "passwordManagerRequestSave",
-                                                                          priv->guid);
-  webkit_user_content_manager_unregister_script_message_handler_in_world (ucm,
-                                                                          "passwordFormFocused",
-                                                                          priv->guid);
-  webkit_user_content_manager_unregister_script_message_handler (ucm, "aboutApps");
-  webkit_user_content_manager_unregister_script_message_handler_in_world (ucm,
-                                                                          "passwordManagerSave",
-                                                                          priv->guid);
+                                                                 "tlsErrorPage",
+                                                                 NULL);
+  webkit_user_content_manager_unregister_script_message_handler (ucm,
+                                                                 "unsafeBrowsingErrorPage",
+                                                                 NULL);
+  webkit_user_content_manager_unregister_script_message_handler (ucm,
+                                                                 "passwordManagerRequestSave",
+                                                                 priv->guid);
+  webkit_user_content_manager_unregister_script_message_handler (ucm,
+                                                                 "passwordFormFocused",
+                                                                 priv->guid);
+  webkit_user_content_manager_unregister_script_message_handler (ucm,
+                                                                 "aboutApps",
+                                                                 NULL);
+  webkit_user_content_manager_unregister_script_message_handler (ucm,
+                                                                 "passwordManagerSave",
+                                                                 priv->guid);
 }
