@@ -1,6 +1,7 @@
 /* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /*
  *  Copyright © 2011 Igalia S.L.
+ *  Copyright © 2022 Matthew Leeds
  *
  *  This file is part of Epiphany.
  *
@@ -23,6 +24,7 @@
 
 #include "ephy-debug.h"
 #include "ephy-file-helpers.h"
+#include "ephy-flatpak-utils.h"
 #include "ephy-profile-utils.h"
 #include "ephy-settings.h"
 
@@ -32,70 +34,51 @@
 #include <stdlib.h>
 #include <string.h>
 #include <fcntl.h>
+#include <libportal-gtk4/portal-gtk4.h>
+#include <glib/gi18n.h>
+#include <gio/gunixoutputstream.h>
 
 /* Web apps are installed in the default data dir of the user. Every
  * app has its own profile directory. To create a web app, an ID needs
- * to be generated using the given name as <normalized-name>-checksum.
+ * to be generated using the checksum of the given name.
  * The ID is used to uniquely identify the app.
  *
  *  - Name: the user-visible pretty app name
  *  - Normalized name: lowercase name
- *  - Checksum: SHA-1 of name
- *  - ID: <normalized-name>-<checksum>
- *  - GApplication ID: see below
- *  - Profile directory: <gapplication-id>
- *  - Desktop file: <profile-dir>/<gapplication-id>.desktop
+ *  - ID: SHA-1 of name
+ *  - Epiphany App ID: org.gnome.Epiphany or org.gnome.Epiphany.Devel or org.gnome.Epiphany.Canary
+ *  - GApplication ID: <epiphany-app-id>.WebApp_<id>
+ *  - Profile directory: <user-data-dir>/<gapplication-id>
+ *  - Desktop file: <user-data-dir>/xdg-desktop-portal/applications/<gapplication-id>.desktop
+ *  - Icon: <user-data-dir>/xdg-desktop-portal/icon/<gapplication-id>.png
+ *  - Sym link: <user-data-dir>/applications/<gapplication-id>.desktop
  *
  * Note that our ID and GApplication ID are different. Yes, this is confusing.
  *
- * For GApplication ID, there are two cases:
- *
- *  - If <id> is safe to use in a D-Bus identifier,
- *    then: GApplication ID is org.gnome.Epiphany.WebApp-<id>
- *  - otherwise: GAppliation ID is org.gnome.Epiphany.WebApp-<checksum>
- *
  * System web applications have a profile dir without a desktop file.
  */
 
-#define EPHY_WEB_APP_GAPPLICATION_ID_PREFIX "org.gnome.Epiphany.WebApp-"
+GQuark webapp_error_quark (void);
+G_DEFINE_QUARK (webapp - error - quark, webapp_error)
+#define WEBAPP_ERROR webapp_error_quark ()
+
+typedef enum {
+  WEBAPP_ERROR_FAILED = 1001,
+  WEBAPP_ERROR_EXISTS = 1002
+} WebappErrorCode;
 
 char *
 ephy_web_application_get_app_id_from_name (const char *name)
 {
-  g_autofree char *normal_id = NULL;
-  g_autofree char *checksum = NULL;
-
-  /* FIXME: We should not use hyphens in app IDs. Sadly, changing this requires
-   * a new migration.
-   */
-  normal_id = g_utf8_strdown (name, -1);
-  g_strdelimit (normal_id, " ", '-');
-  g_strdelimit (normal_id, G_DIR_SEPARATOR_S, '-');
-
-  checksum = g_compute_checksum_for_string (G_CHECKSUM_SHA1, name, -1);
-  return g_strdup_printf ("%s-%s", normal_id, checksum);
-}
-
-static char *
-get_encoded_path (const char *path)
-{
-  g_autofree char *encoded = NULL;
-  g_autoptr (GError) error = NULL;
-
-  encoded = g_filename_from_utf8 (path, -1, NULL, NULL, &error);
-  if (error) {
-    g_warning ("%s", error->message);
-    return NULL;
-  }
-
-  return g_steal_pointer (&encoded);
+  return g_compute_checksum_for_string (G_CHECKSUM_SHA1, name, -1);
 }
 
 static const char *
 get_app_id_from_gapplication_id (const char *name)
 {
   if (!g_str_has_prefix (name, EPHY_WEB_APP_GAPPLICATION_ID_PREFIX)) {
-    g_warning ("GApplication ID %s does not begin with required prefix %s", name, EPHY_WEB_APP_GAPPLICATION_ID_PREFIX);
+    g_warning ("GApplication ID %s does not begin with required prefix %s",
+               name, EPHY_WEB_APP_GAPPLICATION_ID_PREFIX);
     return NULL;
   }
 
@@ -106,47 +89,10 @@ static char *
 get_gapplication_id_from_id (const char *id)
 {
   g_autofree char *gapplication_id = NULL;
-  const char *final_hyphen;
-  const char *checksum;
 
-  /* FIXME: Ideally we would convert hyphens to underscores here, because
-   * hyphens are not very friendly to D-Bus. However, changing this
-   * would require a new profile migration, because the GApplication ID
-   * must exactly match the desktop file basename.
-   *
-   * If we're willing to do another migration in the future, then we
-   * should drop this path and always return just the prefix plus hash,
-   * using underscores rather than hyphens.
-   */
   gapplication_id = g_strconcat (EPHY_WEB_APP_GAPPLICATION_ID_PREFIX, id, NULL);
-  if (g_application_id_is_valid (gapplication_id))
-    return g_steal_pointer (&gapplication_id);
-
-  /* Split ID into: <normalized-name>-<checksum> */
-  final_hyphen = strrchr (id, '-');
-  if (!final_hyphen) {
-    g_warning ("Web app ID %s is broken: must contain a hyphen", id);
-    return NULL;
-  }
-  checksum = final_hyphen + 1;
-
-  if (*checksum == '\0') {
-    g_warning ("Web app ID %s is broken: should end with checksum, not hyphen", id);
-    return NULL;
-  }
-
-  /* We'll simply omit the <normalized-name> from the app ID, in order
-   * to avoid problematic characters. Ideally we would use an underscore
-   * here too, rather than a hyphen, but let's be consistent with
-   * existing web apps.
-   */
-  g_clear_pointer (&gapplication_id, g_free);
-  gapplication_id = g_strconcat (EPHY_WEB_APP_GAPPLICATION_ID_PREFIX, checksum, NULL);
-
-  if (!g_application_id_is_valid (gapplication_id)) {
-    g_warning ("Web app ID %s is broken: derived GApplication ID %s is not a valid app ID (is the final component alphanumeric?)", id, gapplication_id);
-    return NULL;
-  }
+  if (!g_application_id_is_valid (gapplication_id))
+    g_error ("Failed to get GApplication ID from app ID %s", id);
 
   return g_steal_pointer (&gapplication_id);
 }
@@ -154,30 +100,20 @@ get_gapplication_id_from_id (const char *id)
 static char *
 get_app_profile_directory_name (const char *id)
 {
-  g_autofree char *gapplication_id = NULL;
-
-  gapplication_id = get_gapplication_id_from_id (id);
-  if (!gapplication_id)
-    g_error ("Failed to get GApplication ID from app ID %s", id);
-
-  return get_encoded_path (gapplication_id);
+  return get_gapplication_id_from_id (id);
 }
 
 static char *
 get_app_desktop_filename (const char *id)
 {
   g_autofree char *gapplication_id = NULL;
-  g_autofree char *filename = NULL;
 
   /* Warning: the GApplication ID must exactly match the desktop file's
    * basename. Don't overthink this or stuff will break, e.g. GNotification.
    */
   gapplication_id = get_gapplication_id_from_id (id);
-  if (!gapplication_id)
-    g_error ("Failed to get GApplication ID from app ID %s", id);
 
-  filename = g_strconcat (gapplication_id, ".desktop", NULL);
-  return get_encoded_path (filename);
+  return g_strconcat (gapplication_id, ".desktop", NULL);
 }
 
 const char *
@@ -199,7 +135,8 @@ ephy_web_application_get_gapplication_id_from_profile_directory (const char *pro
     name += strlen ("app-");
 
   if (!g_str_has_prefix (name, EPHY_WEB_APP_GAPPLICATION_ID_PREFIX)) {
-    g_warning ("Profile directory %s does not begin with required web app prefix %s", profile_dir, EPHY_WEB_APP_GAPPLICATION_ID_PREFIX);
+    g_warning ("Profile directory %s does not begin with required web app prefix %s",
+               profile_dir, EPHY_WEB_APP_GAPPLICATION_ID_PREFIX);
     return NULL;
   }
 
@@ -256,7 +193,10 @@ ephy_web_application_get_config_directory (const char *id)
 
 /**
  * ephy_web_application_delete:
- * @id: the identifier of the web application do delete
+ * @id: the identifier of the web application to delete
+ * @out_app_found: return location for a #EphyWebAppFound. This will be set to
+ *   %EPHY_WEB_APP_NOT_FOUND if deleting the app failed due to it not being
+ *   installed, and %EPHY_WEB_APP_FOUND otherwise.
  *
  * Deletes all the data associated with a Web Application created by
  * Epiphany.
@@ -264,18 +204,21 @@ ephy_web_application_get_config_directory (const char *id)
  * Returns: %TRUE if the web app was succesfully deleted, %FALSE otherwise
  **/
 gboolean
-ephy_web_application_delete (const char *id)
+ephy_web_application_delete (const char      *id,
+                             EphyWebAppFound *out_app_found)
 {
   g_autofree char *profile_dir = NULL;
   g_autofree char *cache_dir = NULL;
   g_autofree char *config_dir = NULL;
   g_autofree char *desktop_file = NULL;
-  g_autofree char *desktop_path = NULL;
-  g_autoptr (GFile) launcher = NULL;
   g_autoptr (GError) error = NULL;
+  XdpPortal *portal = ephy_get_portal ();
 
   g_assert (id);
 
+  if (out_app_found)
+    *out_app_found = EPHY_WEB_APP_FOUND;
+
   profile_dir = ephy_web_application_get_profile_directory (id);
   if (!profile_dir)
     return FALSE;
@@ -284,6 +227,8 @@ ephy_web_application_delete (const char *id)
    * exist. */
   if (!g_file_test (profile_dir, G_FILE_TEST_IS_DIR)) {
     g_warning ("No application with id '%s' is installed.\n", id);
+    if (out_app_found)
+      *out_app_found = EPHY_WEB_APP_NOT_FOUND;
     return FALSE;
   }
 
@@ -291,7 +236,7 @@ ephy_web_application_delete (const char *id)
     g_warning ("Failed to recursively delete %s: %s", profile_dir, error->message);
     return FALSE;
   }
-  LOG ("Deleted application profile.\n");
+  LOG ("Deleted application profile.");
 
   cache_dir = ephy_web_application_get_cache_directory (id);
   if (g_file_test (cache_dir, G_FILE_TEST_IS_DIR)) {
@@ -299,7 +244,7 @@ ephy_web_application_delete (const char *id)
       g_warning ("Failed to recursively delete %s: %s", cache_dir, error->message);
       return FALSE;
     }
-    LOG ("Deleted application cache directory.\n");
+    LOG ("Deleted application cache directory.");
   }
 
   config_dir = ephy_web_application_get_config_directory (id);
@@ -308,7 +253,7 @@ ephy_web_application_delete (const char *id)
       g_warning ("Failed to recursively delete %s: %s", config_dir, error->message);
       return FALSE;
     }
-    LOG ("Deleted application config directory.\n");
+    LOG ("Deleted application config directory.");
   }
 
   desktop_file = get_app_desktop_filename (id);
@@ -317,45 +262,71 @@ ephy_web_application_delete (const char *id)
     return FALSE;
   }
 
-  desktop_path = g_build_filename (g_get_user_data_dir (), "applications", desktop_file, NULL);
-  if (g_file_test (desktop_path, G_FILE_TEST_IS_SYMLINK)) {
-    launcher = g_file_new_for_path (desktop_path);
-    if (!g_file_delete (launcher, NULL, &error)) {
-      g_warning ("Failed to delete %s: %s", desktop_path, error->message);
-      return FALSE;
-    }
-    LOG ("Deleted application launcher.\n");
+  if (!xdp_portal_dynamic_launcher_uninstall (portal, desktop_file, &error)) {
+    g_warning ("Failed to uninstall desktop file using portal: %s", error->message);
+    return FALSE;
   }
 
+  LOG ("Deleted application launcher %s.", desktop_file);
+
   return TRUE;
 }
 
-static char *
-create_desktop_file (const char *id,
-                     const char *name,
-                     const char *address,
-                     const char *profile_dir,
-                     GdkPixbuf  *icon)
+/**
+ * ephy_web_application_delete_by_desktop_file_id:
+ * @desktop_file_id: the .desktop file name for the web app to be deleted, with
+ *   the extension
+ * @out_app_found: return location for a #EphyWebAppFound. This will be set to
+ *   %EPHY_WEB_APP_NOT_FOUND if deleting the app failed due to it not being
+ *   installed, and %EPHY_WEB_APP_FOUND otherwise.
+ *
+ * Deletes all the data associated with a Web Application created by
+ * Epiphany.
+ *
+ * Returns: %TRUE if the web app was succesfully deleted, %FALSE otherwise
+ **/
+gboolean
+ephy_web_application_delete_by_desktop_file_id (const char      *desktop_file_id,
+                                                EphyWebAppFound *out_app_found)
+{
+  const char *id;
+  g_autofree char *gapp_id = NULL;
+
+  g_assert (desktop_file_id);
+
+  gapp_id = g_strdup (desktop_file_id);
+  if (g_str_has_suffix (desktop_file_id, ".desktop"))
+    gapp_id[strlen (desktop_file_id) - strlen (".desktop")] = '\0';
+
+  id = get_app_id_from_gapplication_id (gapp_id);
+
+  return ephy_web_application_delete (id, out_app_found);
+}
+
+static gboolean
+create_desktop_file (const char  *id,
+                     const char  *address,
+                     const char  *profile_dir,
+                     const char  *install_token,
+                     GError     **error)
 {
   g_autofree char *filename = NULL;
   g_autoptr (GKeyFile) file = NULL;
   g_autofree char *exec_string = NULL;
   g_autofree char *wm_class = NULL;
-  g_autofree char *data = NULL;
-  g_autofree char *desktop_file_path = NULL;
-  g_autofree char *apps_path = NULL;
-  g_autofree char *link_path = NULL;
-  g_autoptr (GFile) link = NULL;
-  g_autoptr (GError) error = NULL;
+  g_autofree char *desktop_entry = NULL;
+  XdpPortal *portal = ephy_get_portal ();
 
   g_assert (profile_dir);
 
   filename = get_app_desktop_filename (id);
-  if (!filename)
-    return NULL;
+  if (!filename) {
+    g_set_error (error, WEBAPP_ERROR, WEBAPP_ERROR_FAILED,
+                 _("Failed to get desktop filename for webapp id %s"), id);
+    return FALSE;
+  }
 
   file = g_key_file_new ();
-  g_key_file_set_value (file, "Desktop Entry", "Name", name);
   exec_string = g_strdup_printf ("epiphany --application-mode \"--profile=%s\" %s",
                                  profile_dir,
                                  address);
@@ -365,43 +336,23 @@ create_desktop_file (const char *id,
   g_key_file_set_value (file, "Desktop Entry", "Type", "Application");
   g_key_file_set_value (file, "Desktop Entry", "Categories", "GNOME;GTK;");
 
-  if (icon) {
-    g_autoptr (GOutputStream) stream = NULL;
-    g_autofree char *path = NULL;
-    g_autoptr (GFile) image = NULL;
-
-    path = g_build_filename (profile_dir, EPHY_WEB_APP_ICON_NAME, NULL);
-    image = g_file_new_for_path (path);
-
-    stream = (GOutputStream *)g_file_create (image, 0, NULL, NULL);
-    gdk_pixbuf_save_to_stream (icon, stream, "png", NULL, NULL, NULL);
-    g_key_file_set_value (file, "Desktop Entry", "Icon", path);
-  }
-
   wm_class = g_strconcat (EPHY_WEB_APP_GAPPLICATION_ID_PREFIX, id, NULL);
   g_key_file_set_value (file, "Desktop Entry", "StartupWMClass", wm_class);
 
   g_key_file_set_value (file, "Desktop Entry", "X-Purism-FormFactor", "Workstation;Mobile;");
 
-  data = g_key_file_to_data (file, NULL, NULL);
+  desktop_entry = g_key_file_to_data (file, NULL, NULL);
 
-  desktop_file_path = g_build_filename (profile_dir, filename, NULL);
-
-  if (!g_file_set_contents (desktop_file_path, data, -1, NULL))
-    g_clear_pointer (&desktop_file_path, g_free);
-
-  /* Create a symlink in XDG_DATA_DIR/applications for the Shell to
-   * pick up this application. */
-  apps_path = g_build_filename (g_get_user_data_dir (), "applications", NULL);
-  if (ephy_ensure_dir_exists (apps_path, &error)) {
-    link_path = g_build_filename (apps_path, filename, NULL);
-    link = g_file_new_for_path (link_path);
-    g_file_make_symbolic_link (link, desktop_file_path, NULL, NULL);
-  } else {
-    g_warning ("Error creating application symlink: %s", error->message);
+  if (!xdp_portal_dynamic_launcher_install (portal, install_token, filename,
+                                            desktop_entry, error)) {
+    g_prefix_error (error, _("Failed to install desktop file %s: "), filename);
+    ephy_file_delete_dir_recursively (profile_dir, NULL);
+    return FALSE;
   }
 
-  return g_steal_pointer (&desktop_file_path);
+  LOG ("Created application launcher %s.", filename);
+
+  return TRUE;
 }
 
 /**
@@ -409,37 +360,39 @@ create_desktop_file (const char *id,
  * @id: the identifier for the new web application
  * @address: the address of the new web application
  * @name: the name for the new web application
- * @icon: the icon for the new web application
+ * @install_token: the install token acquired via portal methods
  * @options: the options for the new web application
+ * @error: return location for a GError pointer
  *
  * Creates a new Web Application for @address.
  *
- * Returns: (transfer-full): the path to the desktop file representing the new application
+ * Returns: %TRUE on success, %FALSE on failure with @error set
  **/
-char *
-ephy_web_application_create (const char                *id,
-                             const char                *address,
-                             const char                *name,
-                             GdkPixbuf                 *icon,
-                             EphyWebApplicationOptions  options)
+gboolean
+ephy_web_application_create (const char                 *id,
+                             const char                 *address,
+                             const char                 *install_token,
+                             EphyWebApplicationOptions   options,
+                             GError                    **error)
 {
   g_autofree char *app_file = NULL;
   g_autofree char *profile_dir = NULL;
-  g_autofree char *desktop_file_path = NULL;
   int fd;
 
   /* If there's already a WebApp profile for the contents of this
    * view, do nothing. */
   profile_dir = ephy_web_application_get_profile_directory (id);
   if (g_file_test (profile_dir, G_FILE_TEST_IS_DIR)) {
-    g_warning ("Profile directory %s already exists", profile_dir);
-    return NULL;
+    g_set_error (error, WEBAPP_ERROR, WEBAPP_ERROR_EXISTS,
+                 _("Profile directory %s already exists"), profile_dir);
+    return FALSE;
   }
 
   /* Create the profile directory, populate it. */
   if (g_mkdir_with_parents (profile_dir, 488) == -1) {
-    g_warning ("Failed to create directory %s", profile_dir);
-    return NULL;
+    g_set_error (error, WEBAPP_ERROR, WEBAPP_ERROR_FAILED,
+                 _("Failed to create directory %s"), profile_dir);
+    return FALSE;
   }
 
   /* Skip migration for new web apps. */
@@ -449,17 +402,19 @@ ephy_web_application_create (const char                *id,
   app_file = g_build_filename (profile_dir, ".app", NULL);
   fd = g_open (app_file, O_WRONLY | O_CREAT | O_TRUNC, 0644);
   if (fd < 0) {
-    g_warning ("Failed to create .app file: %s", g_strerror (errno));
-    return NULL;
+    g_set_error (error, WEBAPP_ERROR, WEBAPP_ERROR_FAILED,
+                 _("Failed to create .app file: %s"), g_strerror (errno));
+    return FALSE;
   }
   close (fd);
 
   /* Create the deskop file. */
-  desktop_file_path = create_desktop_file (id, name, address, profile_dir, icon);
-  if (desktop_file_path)
-    ephy_web_application_initialize_settings (profile_dir, options);
+  if (!create_desktop_file (id, address, profile_dir, install_token, error))
+    return FALSE;
 
-  return g_steal_pointer (&desktop_file_path);
+  ephy_web_application_initialize_settings (profile_dir, options);
+
+  return TRUE;
 }
 
 char *
@@ -520,14 +475,34 @@ ephy_web_application_ensure_for_app_info (GAppInfo *app_info)
   return g_steal_pointer (&profile_dir);
 }
 
+GKeyFile *
+ephy_web_application_get_desktop_keyfile (const char  *id,
+                                          GError     **error)
+{
+  XdpPortal *portal = ephy_get_portal ();
+  g_autofree char *desktop_basename = NULL;
+  g_autofree char *contents = NULL;
+  g_autoptr (GKeyFile) key_file = NULL;
+
+  desktop_basename = get_app_desktop_filename (id);
+  contents = xdp_portal_dynamic_launcher_get_desktop_entry (portal, desktop_basename, error);
+  if (!contents)
+    return NULL;
+
+  key_file = g_key_file_new ();
+  if (!g_key_file_load_from_data (key_file, contents, -1, G_KEY_FILE_NONE, error))
+    return NULL;
+
+  return g_steal_pointer (&key_file);
+}
+
 void
 ephy_web_application_setup_from_profile_directory (const char *profile_directory)
 {
   const char *gapplication_id;
   const char *id;
-  g_autofree char *desktop_basename = NULL;
-  g_autofree char *desktop_filename = NULL;
-  g_autoptr (GDesktopAppInfo) desktop_info = NULL;
+  g_autoptr (GKeyFile) desktop_keyfile = NULL;
+  g_autoptr (GError) error = NULL;
 
   g_assert (profile_directory != NULL);
 
@@ -543,12 +518,18 @@ ephy_web_application_setup_from_profile_directory (const char *profile_directory
     g_error ("Failed to get app ID from GApplication ID %s", gapplication_id);
 
   /* Get display name from desktop file */
-  desktop_basename = get_app_desktop_filename (id);
-  desktop_filename = g_build_filename (profile_directory, desktop_basename, NULL);
-  desktop_info = g_desktop_app_info_new_from_filename (desktop_filename);
-  if (!desktop_info)
-    g_error ("Required desktop file not present at %s", desktop_filename);
-  g_set_application_name (g_app_info_get_name (G_APP_INFO (desktop_info)));
+  desktop_keyfile = ephy_web_application_get_desktop_keyfile (id, &error);
+  if (!desktop_keyfile) {
+    g_warning ("Required desktop file '%s' not available: %s", gapplication_id, error->message);
+    g_clear_error (&error);
+  } else {
+    g_autofree char *name = NULL;
+    name = g_key_file_get_string (desktop_keyfile, "Desktop Entry", "Name", NULL);
+    if (!name)
+      g_warning ("Missing name in desktop file '%s'", gapplication_id);
+    else
+      g_set_application_name (name);
+  }
 }
 
 void
@@ -563,22 +544,104 @@ ephy_web_application_setup_from_desktop_file (GDesktopAppInfo *desktop_info)
   g_set_application_name (g_app_info_get_display_name (app_info));
 }
 
+void
+ephy_web_application_launch (const char *id)
+{
+  XdpPortal *portal = ephy_get_portal ();
+  g_autofree char *desktop_basename = NULL;
+  g_autoptr (GError) error = NULL;
+
+  desktop_basename = get_app_desktop_filename (id);
+  if (!xdp_portal_dynamic_launcher_launch (portal, desktop_basename, NULL, &error))
+    g_warning ("Failed to launch app '%s': %s", desktop_basename, error->message);
+}
+
 void
 ephy_web_application_free (EphyWebApplication *app)
 {
   g_free (app->id);
   g_free (app->name);
-  g_free (app->icon_url);
+  g_free (app->icon_path);
+  g_free (app->tmp_icon_path);
   g_free (app->url);
   g_free (app->desktop_file);
+  g_free (app->desktop_path);
   g_free (app);
 }
 
+static char *
+ephy_web_application_get_tmp_icon_path (const char  *desktop_path,
+                                        GError     **error)
+{
+  XdpPortal *portal = ephy_get_portal ();
+  g_autoptr (GVariant) icon_v = NULL;
+  g_autofree char *icon_format = NULL;
+  g_autofree char *desktop_basename = NULL;
+  g_autofree char *tmp_file_name = NULL;
+  g_autofree char *tmp_file_path = NULL;
+  g_autoptr (GIcon) icon = NULL;
+  g_autoptr (GOutputStream) stream = NULL;
+  GBytes *bytes;
+  gconstpointer bytes_data;
+  gsize bytes_len;
+  int fd;
+
+  g_return_val_if_fail (desktop_path != NULL, NULL);
+
+  /* This function is only useful inside the sandbox since the icon file on the
+   * host is inaccessible in that case.
+   */
+  g_assert (ephy_is_running_inside_sandbox ());
+
+  desktop_basename = g_path_get_basename (desktop_path);
+  icon_v = xdp_portal_dynamic_launcher_get_icon (portal, desktop_basename, &icon_format, NULL, error);
+  if (!icon_v)
+    return NULL;
+
+  tmp_file_name = ephy_file_tmp_filename (".ephy-webapp-icon-XXXXXX", icon_format);
+  tmp_file_path = g_build_filename (ephy_file_tmp_dir (), tmp_file_name, NULL);
+
+  icon = g_icon_deserialize (icon_v);
+  if (!icon) {
+    g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_INVALID_DATA,
+                         "Icon deserialization failed");
+    return NULL;
+  }
+
+  if (!G_IS_BYTES_ICON (icon)) {
+    g_set_error (error, G_IO_ERROR, G_IO_ERROR_INVALID_DATA,
+                 "Unexpected icon type: %s", G_OBJECT_TYPE_NAME (icon));
+    return NULL;
+  }
+
+  bytes = g_bytes_icon_get_bytes (G_BYTES_ICON (icon));
+  fd = g_open (tmp_file_path, O_WRONLY | O_CREAT | O_TRUNC, 0600);
+  if (fd == -1) {
+    g_set_error (error, G_IO_ERROR, g_io_error_from_errno (errno),
+                 "Failed to create tmpfile ‘%s’: %s",
+                 tmp_file_path, g_strerror (errno));
+    return NULL;
+  }
+
+  stream = g_unix_output_stream_new (fd, TRUE);
+  /* Use write_all() instead of write_bytes() so we don't have to worry about
+   * partial writes (https://gitlab.gnome.org/GNOME/glib/-/issues/570).
+   */
+  bytes_data = g_bytes_get_data (bytes, &bytes_len);
+  if (!g_output_stream_write_all (stream, bytes_data, bytes_len, NULL, NULL, error))
+    return NULL;
+
+  if (!g_output_stream_close (stream, NULL, error))
+    return NULL;
+
+  return g_steal_pointer (&tmp_file_path);
+}
+
 EphyWebApplication *
-ephy_web_application_for_profile_directory (const char *profile_dir)
+ephy_web_application_for_profile_directory (const char            *profile_dir,
+                                            EphyWebAppNeedTmpIcon  need_tmp_icon)
 {
   g_autoptr (EphyWebApplication) app = NULL;
-  g_autofree char *desktop_file_path = NULL;
   const char *id;
   g_autoptr (GDesktopAppInfo) desktop_info = NULL;
   const char *exec;
@@ -586,8 +649,8 @@ ephy_web_application_for_profile_directory (const char *profile_dir)
   g_auto (GStrv) argv = NULL;
   g_autoptr (GFile) file = NULL;
   g_autoptr (GFileInfo) file_info = NULL;
-  guint64 created;
-  g_autoptr (GDate) date = NULL;
+  g_autoptr (GError) error = NULL;
+  g_autoptr (GKeyFile) key_file = NULL;
 
   id = get_app_id_from_profile_directory (profile_dir);
   if (!id)
@@ -596,45 +659,82 @@ ephy_web_application_for_profile_directory (const char *profile_dir)
   app = g_new0 (EphyWebApplication, 1);
   app->id = g_strdup (id);
 
-  app->desktop_file = get_app_desktop_filename (id);
-  desktop_file_path = g_build_filename (profile_dir, app->desktop_file, NULL);
-  desktop_info = g_desktop_app_info_new_from_filename (desktop_file_path);
+  app->desktop_path = ephy_web_application_get_desktop_path (id);
+  if (ephy_can_install_web_apps ()) {
+    key_file = ephy_web_application_get_desktop_keyfile (id, &error);
+    if (key_file == NULL) {
+      g_warning ("Failed to get desktop keyfile for id %s from portal: %s", id, error->message);
+      g_clear_pointer (&app, ephy_web_application_free); /* avoid a scan-build warning */
+      return NULL;
+    }
+
+    app->name = g_key_file_get_string (key_file, "Desktop Entry", "Name", NULL);
+    app->icon_path = g_key_file_get_string (key_file, "Desktop Entry", "Icon", NULL);
+
+    if (ephy_is_running_inside_sandbox () && need_tmp_icon == EPHY_WEB_APP_NEED_TMP_ICON) {
+      app->tmp_icon_path = ephy_web_application_get_tmp_icon_path (app->desktop_path, &error);
+      if (!app->tmp_icon_path)
+        g_warning ("Failed to get tmp icon path for app %s: %s", app->id, error->message);
+    }
+
+    exec = g_key_file_get_string (key_file, "Desktop Entry", "Exec", NULL);
+    if (g_shell_parse_argv (exec, &argc, &argv, NULL))
+      app->url = g_strdup (argv[argc - 1]);
+
+    file = g_file_new_for_path (profile_dir);
+
+    /* FIXME: this should use TIME_CREATED but it does not seem to be working. */
+    file_info = g_file_query_info (file, G_FILE_ATTRIBUTE_TIME_MODIFIED, 0, NULL, NULL);
+    app->install_date_uint64 = g_file_info_get_attribute_uint64 (file_info, G_FILE_ATTRIBUTE_TIME_MODIFIED);
+
+    return g_steal_pointer (&app);
+  }
+
+  /* sandbox should take the code path above */
+  if (ephy_is_running_inside_sandbox ()) {
+    g_warning ("Epiphany is sandboxed but the DynamicLauncher portal is unavailable; can't use web app functionality");
+    return NULL;
+  }
+
+  desktop_info = g_desktop_app_info_new_from_filename (app->desktop_path);
   if (!desktop_info) {
     g_clear_pointer (&app, ephy_web_application_free); /* avoid a scan-build warning */
     return NULL;
   }
 
   app->name = g_strdup (g_app_info_get_name (G_APP_INFO (desktop_info)));
-  app->icon_url = g_desktop_app_info_get_string (desktop_info, "Icon");
+  app->icon_path = g_desktop_app_info_get_string (desktop_info, "Icon");
   exec = g_app_info_get_commandline (G_APP_INFO (desktop_info));
   if (g_shell_parse_argv (exec, &argc, &argv, NULL))
     app->url = g_strdup (argv[argc - 1]);
 
-  file = g_file_new_for_path (desktop_file_path);
+  file = g_file_new_for_path (app->desktop_path);
 
   /* FIXME: this should use TIME_CREATED but it does not seem to be working. */
   file_info = g_file_query_info (file, G_FILE_ATTRIBUTE_TIME_MODIFIED, 0, NULL, NULL);
-  created = g_file_info_get_attribute_uint64 (file_info, G_FILE_ATTRIBUTE_TIME_MODIFIED);
-
-  date = g_date_new ();
-  g_date_set_time_t (date, (time_t)created);
-  g_date_strftime (app->install_date, 127, "%x", date);
+  app->install_date_uint64 = g_file_info_get_attribute_uint64 (file_info, G_FILE_ATTRIBUTE_TIME_MODIFIED);
 
   return g_steal_pointer (&app);
 }
 
-static GList *
-ephy_web_application_get_application_list_internal (gboolean only_legacy)
+/**
+ * ephy_web_application_get_application_list:
+ *
+ * Gets a list of the currently installed web applications.
+ * Free the returned GList with
+ * ephy_web_application_free_application_list.
+ *
+ * Returns: (transfer-full): a #GList of #EphyWebApplication objects
+ **/
+GList *
+ephy_web_application_get_application_list (void)
 {
   g_autoptr (GFileEnumerator) children = NULL;
   GList *applications = NULL;
   g_autofree char *parent_directory_path = NULL;
   g_autoptr (GFile) parent_directory = NULL;
 
-  if (only_legacy)
-    parent_directory_path = g_build_filename (g_get_user_config_dir (), "epiphany", NULL);
-  else
-    parent_directory_path = g_strdup (g_get_user_data_dir ());
+  parent_directory_path = g_strdup (g_get_user_data_dir ());
 
   parent_directory = g_file_new_for_path (parent_directory_path);
   children = g_file_enumerate_children (parent_directory,
@@ -651,19 +751,15 @@ ephy_web_application_get_application_list_internal (gboolean only_legacy)
       break;
 
     name = g_file_info_get_name (info);
-    if ((only_legacy && g_str_has_prefix (name, "app-")) ||
-        (!only_legacy && g_str_has_prefix (name, EPHY_WEB_APP_GAPPLICATION_ID_PREFIX))) {
+    if (g_str_has_prefix (name, EPHY_WEB_APP_GAPPLICATION_ID_PREFIX)) {
       g_autoptr (EphyWebApplication) app = NULL;
       g_autofree char *profile_dir = NULL;
 
       profile_dir = g_build_filename (parent_directory_path, name, NULL);
-      app = ephy_web_application_for_profile_directory (profile_dir);
+      app = ephy_web_application_for_profile_directory (profile_dir, EPHY_WEB_APP_NEED_TMP_ICON);
       if (app) {
-        if (!only_legacy) {
-          g_autofree char *app_file = g_build_filename (profile_dir, ".app", NULL);
-          if (g_file_test (app_file, G_FILE_TEST_EXISTS))
-            applications = g_list_prepend (applications, g_steal_pointer (&app));
-        } else
+        g_autofree char *app_file = g_build_filename (profile_dir, ".app", NULL);
+        if (g_file_test (app_file, G_FILE_TEST_EXISTS))
           applications = g_list_prepend (applications, g_steal_pointer (&app));
       }
     }
@@ -672,40 +768,6 @@ ephy_web_application_get_application_list_internal (gboolean only_legacy)
   return g_list_reverse (applications);
 }
 
-/**
- * ephy_web_application_get_application_list:
- *
- * Gets a list of the currently installed web applications.
- * Free the returned GList with
- * ephy_web_application_free_application_list.
- *
- * Returns: (transfer-full): a #GList of #EphyWebApplication objects
- **/
-GList *
-ephy_web_application_get_application_list (void)
-{
-  return ephy_web_application_get_application_list_internal (FALSE);
-}
-
-/**
- * ephy_web_application_get_legacy_application_list:
- *
- * Gets a list of the currently installed web applications.
- * This is only used for the profile migrator as it gets
- * applications in the legacy directory.
- *
- * Free the returned GList with
- * ephy_web_application_free_application_list.
- *
- * Returns: (transfer-full): a #GList of #EphyWebApplication objects
- **/
-GList *
-ephy_web_application_get_legacy_application_list (void)
-{
-  return ephy_web_application_get_application_list_internal (TRUE);
-}
-
-
 /**
  * ephy_web_application_free_application_list:
  * @list: an #EphyWebApplication GList
@@ -718,6 +780,51 @@ ephy_web_application_free_application_list (GList *list)
   g_list_free_full (list, (GDestroyNotify)ephy_web_application_free);
 }
 
+/**
+ * ephy_web_application_get_desktop_id_list:
+ *
+ * Gets a list of the currently installed web applications' .desktop filenames.
+ * This is useful even though we don't have access to the actual .desktop files
+ * when running under Flatpak, because we return it over D-Bus in the
+ * WebAppProvider service.
+ *
+ * Returns: (transfer-full): a %NULL-terminated array of strings
+ **/
+char **
+ephy_web_application_get_desktop_id_list (void)
+{
+  g_autoptr (GFileEnumerator) children = NULL;
+  g_autoptr (GFile) parent_directory = NULL;
+  GPtrArray *desktop_file_ids;
+
+  parent_directory = g_file_new_for_path (g_get_user_data_dir ());
+  children = g_file_enumerate_children (parent_directory,
+                                        "standard::name",
+                                        0, NULL, NULL);
+  if (!children)
+    return NULL;
+
+  desktop_file_ids = g_ptr_array_new_with_free_func (g_free);
+  for (;;) {
+    g_autoptr (GFileInfo) info = g_file_enumerator_next_file (children, NULL, NULL);
+    const char *name;
+
+    if (!info)
+      break;
+
+    name = g_file_info_get_name (info);
+    if (g_str_has_prefix (name, EPHY_WEB_APP_GAPPLICATION_ID_PREFIX)) {
+      g_autofree char *desktop_file_id = NULL;
+      desktop_file_id = g_strconcat (name, ".desktop", NULL);
+      g_ptr_array_add (desktop_file_ids, g_steal_pointer (&desktop_file_id));
+    }
+  }
+
+  g_ptr_array_add (desktop_file_ids, NULL);
+
+  return (char **)g_ptr_array_free (desktop_file_ids, FALSE);
+}
+
 /**
  * ephy_web_application_exists:
  * @id: the potential identifier of the web application
@@ -811,7 +918,8 @@ urls_have_same_origin (const char *a_url,
 gboolean
 ephy_web_application_is_uri_allowed (const char *uri)
 {
-  g_autoptr (EphyWebApplication) webapp = ephy_web_application_for_profile_directory (ephy_profile_dir ());
+  g_autoptr (EphyWebApplication) webapp = ephy_web_application_for_profile_directory (ephy_profile_dir (),
+                                                                                      EPHY_WEB_APP_NO_TMP_ICON);
   const char *scheme;
   g_auto (GStrv) urls = NULL;
   guint i;
@@ -861,15 +969,13 @@ ephy_web_icon_copy_cb (GFile        *file,
 gboolean
 ephy_web_application_save (EphyWebApplication *app)
 {
-  g_autofree char *profile_dir = NULL;
-  g_autofree char *desktop_file_path = NULL;
   g_autofree char *contents = NULL;
   gboolean saved = FALSE;
   g_autoptr (GError) error = NULL;
 
-  profile_dir = ephy_web_application_get_profile_directory (app->id);
-  desktop_file_path = g_build_filename (profile_dir, app->desktop_file, NULL);
-  if (g_file_get_contents (desktop_file_path, &contents, NULL, &error)) {
+  g_assert (!ephy_is_running_inside_sandbox ());
+
+  if (g_file_get_contents (app->desktop_path, &contents, NULL, &error)) {
     g_autoptr (GKeyFile) key = NULL;
     g_autofree char *name = NULL;
     g_autofree char *icon = NULL;
@@ -888,11 +994,11 @@ ephy_web_application_save (EphyWebApplication *app)
     }
 
     icon = g_key_file_get_string (key, "Desktop Entry", "Icon", NULL);
-    if (g_strcmp0 (icon, app->icon_url) != 0) {
+    if (g_strcmp0 (icon, app->icon_path) != 0) {
       g_autoptr (GFile) new_icon = NULL;
       g_autoptr (GFile) old_icon = NULL;
       changed = TRUE;
-      new_icon = g_file_new_for_path (app->icon_url);
+      new_icon = g_file_new_for_path (app->icon_path);
       old_icon = g_file_new_for_path (icon);
       g_file_copy_async (new_icon, old_icon, G_FILE_COPY_OVERWRITE,
                          G_PRIORITY_DEFAULT, NULL, NULL, NULL,
@@ -913,7 +1019,7 @@ ephy_web_application_save (EphyWebApplication *app)
     }
 
     if (changed) {
-      saved = g_key_file_save_to_file (key, desktop_file_path, &error);
+      saved = g_key_file_save_to_file (key, app->desktop_path, &error);
       if (!saved)
         g_warning ("Failed to save desktop file of web application: %s\n", error->message);
     }
@@ -940,3 +1046,17 @@ ephy_web_application_is_system (EphyWebApplication *app)
 
   return g_settings_get_boolean (web_app_settings, EPHY_PREFS_WEB_APP_SYSTEM);
 }
+
+char *
+ephy_web_application_get_desktop_path (const char *id)
+{
+  g_autofree char *desktop_basename = NULL;
+
+  desktop_basename = get_app_desktop_filename (id);
+
+  /* Note: When running under Flatpak, this path is wrong since the
+   * $XDG_DATA_HOME will be different than on the host. Under Flatpak we only
+   * get the basename from this path and ignore the rest.
+   */
+  return g_build_filename (g_get_user_data_dir (), "applications", desktop_basename, NULL);
+}
