@@ -24,7 +24,7 @@
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <string.h>
-#include <webkit2/webkit2.h>
+#include <webkit/webkit.h>
 
 #include "ephy-history-service.h"
 #include "ephy-embed-shell.h"
@@ -54,9 +54,7 @@ G_DEFINE_TYPE (ClearDataView, clear_data_view, EPHY_TYPE_DATA_VIEW)
   WEBKIT_WEBSITE_DATA_DISK_CACHE | \
   WEBKIT_WEBSITE_DATA_OFFLINE_APPLICATION_CACHE | \
   WEBKIT_WEBSITE_DATA_LOCAL_STORAGE | \
-  WEBKIT_WEBSITE_DATA_WEBSQL_DATABASES | \
   WEBKIT_WEBSITE_DATA_INDEXEDDB_DATABASES | \
-  WEBKIT_WEBSITE_DATA_PLUGIN_DATA | \
   WEBKIT_WEBSITE_DATA_HSTS_CACHE | \
   WEBKIT_WEBSITE_DATA_ITP
 
@@ -72,21 +70,20 @@ static const DataEntry data_entries[] = {
   { 0x004, WEBKIT_WEBSITE_DATA_LOCAL_STORAGE, N_("Local storage data") },
   { 0x008, WEBKIT_WEBSITE_DATA_OFFLINE_APPLICATION_CACHE, N_("Offline web application cache") },
   { 0x010, WEBKIT_WEBSITE_DATA_INDEXEDDB_DATABASES, N_("IndexedDB databases") },
-  { 0x020, WEBKIT_WEBSITE_DATA_WEBSQL_DATABASES, N_("WebSQL databases") },
-  { 0x040, WEBKIT_WEBSITE_DATA_PLUGIN_DATA, N_("Plugins data") },
-  { 0x080, WEBKIT_WEBSITE_DATA_HSTS_CACHE, N_("HSTS policies cache") },
-  { 0x100, WEBKIT_WEBSITE_DATA_ITP, N_("Intelligent Tracking Prevention data") }
+  { 0x020, WEBKIT_WEBSITE_DATA_HSTS_CACHE, N_("HSTS policies cache") },
+  { 0x040, WEBKIT_WEBSITE_DATA_ITP, N_("Intelligent Tracking Prevention data") }
 };
 
 static WebKitWebsiteDataManager *
 get_website_data_manger (void)
 {
-  WebKitWebContext *web_context;
+  WebKitNetworkSession *network_session;
 
-  web_context = ephy_embed_shell_get_web_context (ephy_embed_shell_get_default ());
-  return webkit_web_context_get_website_data_manager (web_context);
+  network_session = ephy_embed_shell_get_network_session (ephy_embed_shell_get_default ());
+  return webkit_network_session_get_website_data_manager (network_session);
 }
 
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
 static void
 website_data_fetched_cb (WebKitWebsiteDataManager *manager,
                          GAsyncResult             *result,
@@ -392,6 +389,7 @@ row_visible_func (GtkTreeModel  *model,
 
   return visible;
 }
+G_GNUC_END_IGNORE_DEPRECATIONS
 
 static void
 clear_data_view_dispose (GObject *object)
@@ -427,6 +425,7 @@ clear_data_view_class_init (ClearDataViewClass *klass)
   gtk_widget_class_bind_template_callback (widget_class, search_text_changed_cb);
 }
 
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
 static void
 clear_data_view_init (ClearDataView *clear_data_view)
 {
@@ -447,3 +446,4 @@ clear_data_view_init (ClearDataView *clear_data_view)
                                      (GAsyncReadyCallback)website_data_fetched_cb,
                                      clear_data_view);
 }
+G_GNUC_END_IGNORE_DEPRECATIONS
